@@ -34,19 +34,15 @@ from duplicity import log
 from duplicity import path
 from duplicity import selection
 
-# Also import the sshbackend module specifically because we stomp on
-# its options.
-from duplicity.backends import sshbackend
-from duplicity.backends import imapbackend
-
 
 select_opts = [] # Will hold all the selection options
 select_files = [] # Will hold file objects when filelist given
-full_backup = None # Will be set to true if -f or --full option given
-list_current = None # Will be set to true if --list-current option given
-collection_status = None # Will be set to true if --collection-status given
-cleanup = None # Set to true if --cleanup option given
-verify = None # Set to true if --verify option given
+
+full_backup = None # Will be set to true if full command given
+list_current = None # Will be set to true if list-current command given
+collection_status = None # Will be set to true if collection-status command given
+cleanup = None # Set to true if cleanup command given
+verify = None # Set to true if verify command given
 
 commands = ["cleanup",
             "collection-status",
@@ -73,6 +69,7 @@ options = ["allow-source-mismatch",
            "exclude-filelist-stdin",
            "exclude-other-filesystems",
            "exclude-regexp=",
+           "fail-on-volume=",
            "file-to-restore=",
            "force",
            "ftp-passive",
@@ -113,11 +110,17 @@ options = ["allow-source-mismatch",
            "volsize=",
            ]
 
+
 def old_fn_deprecation(opt):
     print >>sys.stderr, _("Warning: Option %s is pending deprecation "
                           "and will be removed in a future release.\n"
                           "Use of default filenames is strongly suggested.") % opt
 
+
+def expand_fn(filename):
+    return os.path.expanduser(os.path.expandvars(filename))
+
+
 def parse_cmdline_options(arglist):
     """Parse argument list"""
     global select_opts, select_files, full_backup
@@ -131,9 +134,6 @@ def parse_cmdline_options(arglist):
             log.FatalError(_("Error opening file %s") % filename,
                            log.ErrorCode.cant_open_filelist)
 
-    def expand_fn(filename):
-        return os.path.expanduser(os.path.expandvars(filename))
-
     # expect no cmd and two positional args
     cmd = ""
     num_expect = 2
@@ -199,11 +199,11 @@ def parse_cmdline_options(arglist):
         if opt == "--allow-source-mismatch":
             globals.allow_source_mismatch = 1
         elif opt == "--archive-dir":
-            set_archive_dir(expand_fn(arg))
+            globals.archive_dir = arg
         elif opt == "--asynchronous-upload":
             globals.async_concurrency = 1 # (yes 1, this is not a boolean)
         elif opt == "--current-time":
-            dup_time.setcurtime(get_int(arg, "current-time"))
+            dup_time.setcurtime(get_int(arg, opt))
         elif opt == "--dry-run":
             globals.dry_run = True
         elif opt == "--encrypt-key":
@@ -229,6 +229,8 @@ def parse_cmdline_options(arglist):
         elif opt == "--exclude-filelist-stdin":
             select_opts.append(("--exclude-filelist", "standard input"))
             select_files.append(sys.stdin)
+        elif opt == "--fail-on-volume":
+            globals.fail_on_volume = get_int(arg, opt)
         elif opt == "--full-if-older-than":
             globals.full_force_time = dup_time.genstrtotime(arg)
         elif opt == "--force":
@@ -238,7 +240,7 @@ def parse_cmdline_options(arglist):
         elif opt == "--ftp-regular":
             globals.ftp_connection = 'regular'
         elif opt == "--imap-mailbox":
-            imapbackend.imap_mailbox = arg.strip()
+            globals.imap_mailbox = arg.strip()
         elif opt == "--gpg-options":
             gpg.gpg_options = (gpg.gpg_options + ' ' + arg).strip()
         elif opt in ["-h", "--help"]:
@@ -248,7 +250,7 @@ def parse_cmdline_options(arglist):
             select_opts.append(("--include-filelist", "standard input"))
             select_files.append(sys.stdin)
         elif opt == "--log-fd":
-            log_fd = int(arg)
+            log_fd = get_int(arg, opt)
             if log_fd < 1:
                 command_line_error("log-fd must be greater than zero.")
             try:
@@ -268,7 +270,7 @@ def parse_cmdline_options(arglist):
         elif opt == "--null-separator":
             globals.null_separator = 1
         elif opt == "--num-retries":
-            globals.num_retries = int(arg)
+            globals.num_retries = get_int(arg, opt)
         elif opt == "--old-filenames":
             globals.old_filenames = True
             old_fn_deprecation(opt)
@@ -281,22 +283,22 @@ def parse_cmdline_options(arglist):
         elif opt == "--s3-use-new-style":
             globals.s3_use_new_style = True
         elif opt == "--scp-command":
-            sshbackend.scp_command = arg
+            globals.scp_command = arg
         elif opt == "--sftp-command":
-            sshbackend.sftp_command = arg
+            globals.sftp_command = arg
         elif opt == "--short-filenames":
             globals.short_filenames = 1
             old_fn_deprecation(opt)
         elif opt == "--sign-key":
             set_sign_key(arg)
         elif opt == "--ssh-askpass":
-            sshbackend.ssh_askpass = True
+            globals.ssh_askpass = True
         elif opt == "--ssh-options":
-            sshbackend.ssh_options = (sshbackend.ssh_options + ' ' + arg).strip()
+            globals.ssh_options = (globals.ssh_options + ' ' + arg).strip()
         elif opt == "--tempdir":
             globals.temproot = arg
         elif opt == "--timeout":
-            globals.timeout = int(arg)
+            globals.timeout = get_int(arg, opt)
         elif opt == "--time-separator":
             if arg == '-':
                 command_line_error("Dash ('-') not valid for time-separator.")
@@ -321,7 +323,7 @@ def parse_cmdline_options(arglist):
             elif arg in ['d', 'debug']:
                 verb = log.DEBUG
             elif arg.isdigit() and (len(arg) == 1):
-                verb = int(arg)
+                verb = get_int(arg, opt)
             else:
                 command_line_error("\nVerbosity must be one of: digit [0-9], character [ewnid],\n"
                                    "or word ['error', 'warning', 'notice', 'info', 'debug'].\n"
@@ -329,7 +331,7 @@ def parse_cmdline_options(arglist):
                                    "that verbosity level is set at 2 (Warning) or higher.")
             log.setverbosity(verb)
         elif opt == "--volsize":
-            globals.volsize = int(arg)*1024*1024
+            globals.volsize = get_int(arg, opt)*1024*1024
         elif opt == "--imap-full-address":
             globals.imap_full_address = True
         else:
@@ -342,14 +344,24 @@ def parse_cmdline_options(arglist):
     if len(args) != num_expect:
         command_line_error("Expected %d args, got %d" % (num_expect, len(args)))
 
+    # expand pathname args, but not URL
+    for loc in range(len(args)):
+        if not '://' in args[loc]:
+            args[loc] = expand_fn(args[loc])
+
+    # set and expand archive dir
+    set_archive_dir(expand_fn(globals.archive_dir))
+
     return args
 
+
 def command_line_error(message):
     """Indicate a command line error and exit"""
     log.FatalError(_("Command line error: %s") % (message,) + "\n" +
                    _("Enter 'duplicity --help' for help screen."),
                    log.ErrorCode.command_line)
 
+
 def usage():
     """Print terse usage info"""
     sys.stdout.write(_("""
@@ -365,17 +377,19 @@ Usage:
     duplicity remove-all-but-n-full count [options] target_url
 
 Backends and their URL formats:
-    ssh://user[:password]@other.host[:port]/some_dir
-    scp://user[:password]@other.host[:port]/some_dir
+    cf+http://container_name
+    file:///some_dir
     ftp://user[:password]@other.host[:port]/some_dir
     hsi://user[:password]@other.host[:port]/some_dir
-    file:///some_dir
     imap://user[:password]@other.host[:port]/some_dir
     rsync://user[:password]@other.host[:port]::/module/some_dir
     rsync://user[:password]@other.host[:port]/relative_path
     rsync://user[:password]@other.host[:port]//absolute_path
     s3://other.host/bucket_name[/prefix]
     s3+http://bucket_name[/prefix]
+    scp://user[:password]@other.host[:port]/some_dir
+    ssh://user[:password]@other.host[:port]/some_dir
+    tahoe://alias/directory
     webdav://user[:password]@other.host/some_dir
     webdavs://user[:password]@other.host/some_dir
 
@@ -450,17 +464,23 @@ def get_int(int_string, description):
         return int(int_string)
     except ValueError:
         command_line_error("Received '%s' for %s, need integer" %
-                                          (int_string, description))
+                                          (int_string, description.lstrip('-')))
 
 def set_archive_dir(dirstring):
     """Check archive dir and set global"""
-    archive_dir = path.Path(os.path.expanduser(dirstring))
+    if not os.path.exists(dirstring):
+        try:
+            os.makedirs(dirstring)
+        except:
+            pass
+    archive_dir = path.Path(dirstring)
     if not archive_dir.isdir():
         log.FatalError(_("Specified archive directory '%s' does not exist, "
                          "or is not a directory") % (archive_dir.name,),
                        log.ErrorCode.bad_archive_dir)
     globals.archive_dir = archive_dir
 
+
 def set_sign_key(sign_key):
     """Set globals.sign_key assuming proper key given"""
     if not len(sign_key) == 8 or not re.search("^[0-9A-F]*$", sign_key):
@@ -469,6 +489,7 @@ def set_sign_key(sign_key):
                        log.ErrorCode.bad_sign_key)
     globals.gpg_profile.sign_key = sign_key
 
+
 def set_selection():
     """Return selection iter starting at filename with arguments applied"""
     global select_opts, select_files
@@ -476,6 +497,7 @@ def set_selection():
     sel.ParseArgs(select_opts, select_files)
     globals.select = sel.set_iter()
 
+
 def set_backend(arg1, arg2):
     """Figure out which arg is url, set backend
 
@@ -499,6 +521,7 @@ page for more information.""")
         globals.backend = backend2
         return (1, arg1)
 
+
 def process_local_dir(action, local_pathname):
     """Check local directory, set globals.local_path"""
     local_path = path.Path(path.Path(local_pathname).get_canonical())
@@ -521,6 +544,7 @@ def process_local_dir(action, local_pathname):
 
     globals.local_path = local_path
 
+
 def check_consistency(action):
     """Final consistency check, see if something wrong with command line"""
     global full_backup, select_opts, list_current
@@ -555,6 +579,7 @@ def check_consistency(action):
             command_line_error("restore option incompatible with %s backup"
                                % (action,))
 
+
 def ProcessCommandLine(cmdline_list):
     """Process command line, set globals, return action
 
